If (Not @Compiled) Then
	MsgBox(0, "Error", "This script must be compiled to run correctly.")
	Exit
EndIf

#include <Math.au3>
#include <GDIPlus.au3>
#include <GuiListBox.au3>
#include <GUIConstantsEx.au3>
#include <StaticConstants.au3>
#include <WindowsConstants.au3>

; Hard-coded filenames
Global $LogFile = @ScriptDir & "\data\log.txt"
Global $SaveData = @ScriptDir & "\data\data.ini"
Global $ImageList = @ScriptDir & "\data\list.txt"

; Hard-coded constants
Global $ViewFolder = @ScriptDir & "\view"
Global $FilterFolder = @ScriptDir & "\filter"
Global $Black = 4278190080
Global $White = 4294967295
Global $SS_CENTER_XY = BitOR($SS_CENTER, $SS_CENTERIMAGE)
Global $LogLabel_Restart = "#RESTART"
Global $LogLabel_Exit = "#EXIT"

; Global variables
Global $SleepCounter = 0
Global $SleepIndex = 0
Global $ScriptLocked = False
Global $Restart = False

; Load settings from INI
Global $Threshold_Alpha = IniRead($SaveData, "Filter", "iThresholdAlpha", 100)
Global $Threshold_Color = IniRead($SaveData, "Filter", "iThresholdColor", 100)
Global $Filter_W = IniRead($SaveData, "Filter", "iFilterW", 500)
Global $Filter_H = IniRead($SaveData, "Filter", "iFilterH", 500)
Global $SleepIndex = IniRead($SaveData, "Draw", "iSleepIndex", 10)
Global $SleepInterval = IniRead($SaveData, "Draw", "iSleepInterval", 10)
Global $PathString = IniRead($SaveData, "Draw", "sPathString", "12345678")
Global $MixPath = StringToBool(IniRead($SaveData, "Draw", "bMixPath", "False"))
Global $RotatePath = StringToBool(IniRead($SaveData, "Draw", "bRotatePath", "False"))
Global $Radius = IniRead($SaveData, "Draw", "iRadius", 0)
Global $HotKey_GetMouse = IniRead($SaveData, "HotKeys", "sGetMouse", "{NUMPAD1}")
Global $HotKey_StartDraw = IniRead($SaveData, "HotKeys", "sStartDraw", "{NUMPAD2}")
Global $HotKey_Shutdown = IniRead($SaveData, "HotKeys", "sShutdown", "{NUMPAD3}")
Global $UseMousePos = StringToBool(IniRead($SaveData, "MousePos", "bUseMousePos", "False"))
Global $PosAX = IniRead($SaveData, "MousePos", "iPosAX", 0)
Global $PosAY = IniRead($SaveData, "MousePos", "iPosAY", 0)
Global $PosBX = IniRead($SaveData, "MousePos", "iPosBX", 0)
Global $PosBY = IniRead($SaveData, "MousePos", "iPosBY", 0)

; Load GUI sizes from INI
$GUI_Full_W = IniRead($SaveData, "WindowSize", "iFull_W", 800)
$GUI_Full_H = IniRead($SaveData, "WindowSize", "iFull_H", 600)
$GUI_Panel_W = $GUI_Full_W
$GUI_Panel_H = IniRead($SaveData, "WindowSize", "iPanel_H", 200)
$GUI_Progress_W = $GUI_Full_W
$GUI_Progress_H = IniRead($SaveData, "WindowSize", "iProgress_H", 20)
$GUI_Image_W = $GUI_Full_W
$GUI_Image_H = $GUI_Full_H - $GUI_Panel_H - $GUI_Progress_H

; Open log file, append if script was restarted
Global $LogHandle = FileOpen($LogFile, 0)
$LastLine = FileReadLine($LogHandle, -1)
$LogMode = 2
If ($LastLine = $LogLabel_Restart) Then
	$LogMode = 1
EndIf
FileClose($LogHandle)
$LogHandle = FileOpen($LogFile, $LogMode)

; Start GDIPlus to perform various operations
_GDIPlus_Startup()

; Main GUI window
Global $GUI_Main = GUICreate("Image Draw", $GUI_Full_W, $GUI_Full_H)

; Emply list for images
Global $GUI_Panel_List = GUICtrlCreateList("", GUI_Panel_X(0.0), GUI_Panel_Y(0.0), GUI_Panel_X(0.3), GUI_Panel_Y(1.0), BitOR($WS_BORDER, $WS_VSCROLL), $WS_EX_LAYOUTRTL)

; Buttons to operate list
Global $GUI_Panel_Button_Add = GUICtrlCreateButton("Add", GUI_Panel_X(0.3), GUI_Panel_Y(0.0), GUI_Panel_X(0.1), GUI_Panel_Y(0.2))
Global $GUI_Panel_Button_Up = GUICtrlCreateButton("Up", GUI_Panel_X(0.3), GUI_Panel_Y(0.2), GUI_Panel_X(0.05), GUI_Panel_Y(0.2))
Global $GUI_Panel_Button_Down = GUICtrlCreateButton("Down", GUI_Panel_X(0.35), GUI_Panel_Y(0.2), GUI_Panel_X(0.05), GUI_Panel_Y(0.2))
Global $GUI_Panel_Button_Remove = GUICtrlCreateButton("Remove", GUI_Panel_X(0.3), GUI_Panel_Y(0.4), GUI_Panel_X(0.1), GUI_Panel_Y(0.2))
Global $GUI_Panel_Button_Clear = GUICtrlCreateButton("Clear", GUI_Panel_X(0.3), GUI_Panel_Y(0.6), GUI_Panel_X(0.1), GUI_Panel_Y(0.2))

; Labels for filter settings
GUICtrlCreateLabel("FILTER SETTINGS", GUI_Panel_X(0.4), GUI_Panel_Y(0.0), GUI_Panel_X(0.2), GUI_Panel_Y(0.2), $SS_CENTER_XY)
GUICtrlCreateLabel("Threshold A/C", GUI_Panel_X(0.4), GUI_Panel_Y(0.2), GUI_Panel_X(0.1), GUI_Panel_Y(0.1), $SS_CENTER_XY)
GUICtrlCreateLabel("Filter W/H", GUI_Panel_X(0.4), GUI_Panel_Y(0.3), GUI_Panel_X(0.1), GUI_Panel_Y(0.1), $SS_CENTER_XY)

; Filter settings
Global $GUI_Panel_Input_Threshold_Alpha = GUICtrlCreateInput($Threshold_Alpha, GUI_Panel_X(0.5), GUI_Panel_Y(0.2), GUI_Panel_X(0.05), GUI_Panel_Y(0.1))
Global $GUI_Panel_Input_Threshold_Color = GUICtrlCreateInput($Threshold_Color, GUI_Panel_X(0.55), GUI_Panel_Y(0.2), GUI_Panel_X(0.05), GUI_Panel_Y(0.1))
Global $GUI_Panel_Input_FilterW = GUICtrlCreateInput($Filter_W, GUI_Panel_X(0.5), GUI_Panel_Y(0.3), GUI_Panel_X(0.05), GUI_Panel_Y(0.1))
Global $GUI_Panel_Input_FilterH = GUICtrlCreateInput($Filter_H, GUI_Panel_X(0.55), GUI_Panel_Y(0.3), GUI_Panel_X(0.05), GUI_Panel_Y(0.1))
Global $GUI_Panel_Button_Filter = GUICtrlCreateButton("Filter", GUI_Panel_X(0.5), GUI_Panel_Y(0.4), GUI_Panel_X(0.1), GUI_Panel_Y(0.2))
Global $GUI_Panel_Button_View = GUICtrlCreateButton("View", GUI_Panel_X(0.5), GUI_Panel_Y(0.6), GUI_Panel_X(0.1), GUI_Panel_Y(0.2))

; Labels for hotkeys settings
GUICtrlCreateLabel("HOTKEYS SETTINGS", GUI_Panel_X(0.6), GUI_Panel_Y(0.0), GUI_Panel_X(0.2), GUI_Panel_Y(0.2), $SS_CENTER_XY)
GUICtrlCreateLabel("Get mouse", GUI_Panel_X(0.6), GUI_Panel_Y(0.2), GUI_Panel_X(0.1), GUI_Panel_Y(0.2), $SS_CENTER_XY)
GUICtrlCreateLabel("Start draw", GUI_Panel_X(0.6), GUI_Panel_Y(0.4), GUI_Panel_X(0.1), GUI_Panel_Y(0.2), $SS_CENTER_XY)
GUICtrlCreateLabel("Shutdown", GUI_Panel_X(0.6), GUI_Panel_Y(0.6), GUI_Panel_X(0.1), GUI_Panel_Y(0.2), $SS_CENTER_XY)

; Hotkeys settings
Global $GUI_Panel_Input_Hotkey_GetMouse = GUICtrlCreateInput($HotKey_GetMouse, GUI_Panel_X(0.7), GUI_Panel_Y(0.2), GUI_Panel_X(0.1), GUI_Panel_Y(0.2))
Global $GUI_Panel_Input_HotKey_StartDraw = GUICtrlCreateInput($HotKey_StartDraw, GUI_Panel_X(0.7), GUI_Panel_Y(0.4), GUI_Panel_X(0.1), GUI_Panel_Y(0.2))
Global $GUI_Panel_Input_Hotkey_Shutdown = GUICtrlCreateInput($HotKey_Shutdown, GUI_Panel_X(0.7), GUI_Panel_Y(0.6), GUI_Panel_X(0.1), GUI_Panel_Y(0.2))

; Labels for draw settings
GUICtrlCreateLabel("DRAW SETTINGS", GUI_Panel_X(0.8), GUI_Panel_Y(0.0), GUI_Panel_X(0.2), GUI_Panel_Y(0.2), $SS_CENTER_XY)
GUICtrlCreateLabel("S.Index", GUI_Panel_X(0.8), GUI_Panel_Y(0.2), GUI_Panel_X(0.1), GUI_Panel_Y(0.1), $SS_CENTER_XY)
GUICtrlCreateLabel("S.Interval", GUI_Panel_X(0.8), GUI_Panel_Y(0.3), GUI_Panel_X(0.1), GUI_Panel_Y(0.1), $SS_CENTER_XY)
GUICtrlCreateLabel("Path string", GUI_Panel_X(0.8), GUI_Panel_Y(0.4), GUI_Panel_X(0.1), GUI_Panel_Y(0.1), $SS_CENTER_XY)
GUICtrlCreateLabel("Mix path", GUI_Panel_X(0.8), GUI_Panel_Y(0.5), GUI_Panel_X(0.1), GUI_Panel_Y(0.1), $SS_CENTER_XY)
GUICtrlCreateLabel("Rotate path", GUI_Panel_X(0.8), GUI_Panel_Y(0.6), GUI_Panel_X(0.1), GUI_Panel_Y(0.1), $SS_CENTER_XY)
GUICtrlCreateLabel("Radius", GUI_Panel_X(0.8), GUI_Panel_Y(0.7), GUI_Panel_X(0.1), GUI_Panel_Y(0.1), $SS_CENTER_XY)

; Draw settings
Global $GUI_Panel_Input_SleepIndex = GUICtrlCreateInput($SleepIndex, GUI_Panel_X(0.9), GUI_Panel_Y(0.2), GUI_Panel_X(0.1), GUI_Panel_Y(0.1))
Global $GUI_Panel_Input_SleepInterval = GUICtrlCreateInput($SleepInterval, GUI_Panel_X(0.9), GUI_Panel_Y(0.3), GUI_Panel_X(0.1), GUI_Panel_Y(0.1))
Global $GUI_Panel_Input_PathString = GUICtrlCreateInput($PathString, GUI_Panel_X(0.9), GUI_Panel_Y(0.4), GUI_Panel_X(0.1), GUI_Panel_Y(0.1))
Global $GUI_Panel_Checkbox_MixPath = GUICtrlCreateCheckbox("", GUI_Panel_X(0.9), GUI_Panel_Y(0.5), GUI_Panel_X(0.1), GUI_Panel_Y(0.1))
GUICtrlSetState($GUI_Panel_Checkbox_MixPath, BoolToChecked($MixPath))
Global $GUI_Panel_Checkbox_RotatePath = GUICtrlCreateCheckbox("", GUI_Panel_X(0.9), GUI_Panel_Y(0.6), GUI_Panel_X(0.1), GUI_Panel_Y(0.1))
GUICtrlSetState($GUI_Panel_Checkbox_RotatePath, BoolToChecked($RotatePath))
Global $GUI_Panel_Input_Radius = GUICtrlCreateInput($Radius, GUI_Panel_X(0.9), GUI_Panel_Y(0.7), GUI_Panel_X(0.1), GUI_Panel_Y(0.1))

; Draw buttons
Global $GUI_Panel_Button_DrawOne = GUICtrlCreateButton("Draw one", GUI_Panel_X(0.8), GUI_Panel_Y(0.8), GUI_Panel_X(0.1), GUI_Panel_Y(0.2))
Global $GUI_Panel_Button_DrawAll = GUICtrlCreateButton("Draw all", GUI_Panel_X(0.9), GUI_Panel_Y(0.8), GUI_Panel_X(0.1), GUI_Panel_Y(0.2))

; Mouse positions (cannot be changed from GUI)
GUICtrlCreateLabel("Position A", GUI_Panel_X(0.4), GUI_Panel_Y(0.8), GUI_Panel_X(0.1), GUI_Panel_Y(0.1), $SS_CENTER_XY)
Global $GUI_Panel_Label_PosAX = GUICtrlCreateLabel($PosAX, GUI_Panel_X(0.5), GUI_Panel_Y(0.8), GUI_Panel_X(0.05), GUI_Panel_Y(0.1), $SS_CENTER_XY)
Global $GUI_Panel_Label_PosAY = GUICtrlCreateLabel($PosAY, GUI_Panel_X(0.55), GUI_Panel_Y(0.8), GUI_Panel_X(0.05), GUI_Panel_Y(0.1), $SS_CENTER_XY)
GUICtrlCreateLabel("Position B", GUI_Panel_X(0.4), GUI_Panel_Y(0.9), GUI_Panel_X(0.1), GUI_Panel_Y(0.1), $SS_CENTER_XY)
Global $GUI_Panel_Label_PosBX = GUICtrlCreateLabel($PosBX, GUI_Panel_X(0.5), GUI_Panel_Y(0.9), GUI_Panel_X(0.05), GUI_Panel_Y(0.1), $SS_CENTER_XY)
Global $GUI_Panel_Label_PosBY = GUICtrlCreateLabel($PosBY, GUI_Panel_X(0.55), GUI_Panel_Y(0.9), GUI_Panel_X(0.05), GUI_Panel_Y(0.1), $SS_CENTER_XY)
Global $GUI_Panel_Checkbox_UseMousePos = GUICtrlCreateCheckbox("Use existing mouse positions", GUI_Panel_X(0.6), GUI_Panel_Y(0.8), GUI_Panel_X(0.2), GUI_Panel_Y(0.2))
GUICtrlSetState($GUI_Panel_Checkbox_UseMousePos, BoolToChecked($UseMousePos))

; Image, will be filled further
Global $GUI_Image = 0

; Progress bar
Global $GUI_Progress = GUICtrlCreateProgress(0, $GUI_Full_H - $GUI_Progress_H, $GUI_Full_W, $GUI_Progress_H)

; Show GUI and fill list with last images
GUISetState(@SW_SHOW, $GUI_Main)
LoadListData($GUI_Panel_List)

; Listen for any GUI message
While True
	Switch GUIGetMsg()
		Case $GUI_EVENT_CLOSE
			ExitLoop
		Case $GUI_Panel_List
			ProcessPanelList()
		Case $GUI_Panel_Button_Add
			ProcessPanelButtonAdd()
		Case $GUI_Panel_Button_Remove
			ProcessPanelButtonRemove()
		Case $GUI_Panel_Button_Up
			ProcessPanelButtonUp()
		Case $GUI_Panel_Button_Down
			ProcessPanelButtonDown()
		Case $GUI_Panel_Button_Clear
			ProcessPanelButtonClear()
		Case	$GUI_Panel_Input_Threshold_Alpha, _
				$GUI_Panel_Input_Threshold_Color, _
				$GUI_Panel_Input_FilterW, _
				$GUI_Panel_Input_FilterH, _
				$GUI_Panel_Input_SleepIndex, _
				$GUI_Panel_Input_SleepInterval, _
				$GUI_Panel_Input_PathString, _
				$GUI_Panel_Checkbox_MixPath, _
				$GUI_Panel_Checkbox_RotatePath, _
				$GUI_Panel_Input_Radius, _
				$GUI_Panel_Input_Hotkey_GetMouse, _
				$GUI_Panel_Input_HotKey_StartDraw, _
				$GUI_Panel_Input_Hotkey_Shutdown, _
				$GUI_Panel_Checkbox_UseMousePos
			UpdateSettings()
			SaveSettings()
		Case $GUI_Panel_Button_Filter
			RegisterShutdownWithRestart()
			ProcessPanelButtonFilter()
			UnregisterShutdownWithRestart()
		Case $GUI_Panel_Button_View
			ProcessPanelButtonView()
		Case $GUI_Panel_Button_DrawOne
			RegisterShutdownWithRestart()
			ProcessPanelButtonDrawOne()
			UnregisterShutdownWithRestart()
		Case $GUI_Panel_Button_DrawAll
			RegisterShutdownWithRestart()
			ProcessPanelButtonDrawAll()
			UnregisterShutdownWithRestart()
	EndSwitch
WEnd

; GUIGetMsg = $GUI_EVENT_CLOSE, loop is over, exit
ProcessShutdown()

; Script functions / Start
Func AddLog($Data = "", $Exit = False)
	FileWrite($LogHandle, $Data & @CRLF)
	If ($Exit) Then
		ProcessShutdown()
	EndIf
EndFunc

Func StringToBool($String)
	If ($String = "true") Or ($String = "1") Then
		Return True
	ElseIf ($String = "false") Or ($String = "0") Then
		Return False
	EndIf
	Return False
EndFunc

Func BoolToString($Bool)
	If ($Bool) Then
		Return "True"
	EndIf
	Return "False"
EndFunc

Func BoolToChecked($Bool)
	If ($Bool) Then
		Return $GUI_CHECKED
	EndIf
	Return $GUI_UNCHECKED
EndFunc

Func CheckedToBool($Checkbox)
	If (GUICtrlRead($Checkbox) = $GUI_CHECKED) Then
		Return True
	EndIf
	Return False
EndFunc

Func LockScript()
	$ScriptLocked = True
	While ($ScriptLocked)
		Sleep(100)
	WEnd
EndFunc

Func UnlockScript()
	$ScriptLocked = False
EndFunc

Func ProcessSleep()
	$SleepCounter += 1
	If ($SleepCounter = $SleepIndex) Then
		$SleepCounter = 0
		Sleep($SleepInterval)
	EndIf
EndFunc

Func UpdateSettings()
	$Threshold_Alpha = GUICtrlRead($GUI_Panel_Input_Threshold_Alpha)
	$Threshold_Color = GUICtrlRead($GUI_Panel_Input_Threshold_Color)
	$Filter_W = GUICtrlRead($GUI_Panel_Input_FilterW)
	$Filter_H = GUICtrlRead($GUI_Panel_Input_FilterH)
	$SleepIndex = GUICtrlRead($GUI_Panel_Input_SleepIndex)
	$SleepInterval = GUICtrlRead($GUI_Panel_Input_SleepInterval)
	$PathString = GUICtrlRead($GUI_Panel_Input_PathString)
	$MixPath = CheckedToBool($GUI_Panel_Checkbox_MixPath)
	$RotatePath = CheckedToBool($GUI_Panel_Checkbox_RotatePath)
	$Radius = GUICtrlRead($GUI_Panel_Input_Radius)
	$HotKey_GetMouse = GUICtrlRead($GUI_Panel_Input_Hotkey_GetMouse)
	$HotKey_StartDraw = GUICtrlRead($GUI_Panel_Input_HotKey_StartDraw)
	$HotKey_Shutdown = GUICtrlRead($GUI_Panel_Input_Hotkey_Shutdown)
	$UseMousePos = CheckedToBool($GUI_Panel_Checkbox_UseMousePos)
	$PosAX = GUICtrlRead($GUI_Panel_Label_PosAX)
	$PosAY = GUICtrlRead($GUI_Panel_Label_PosAY)
	$PosBX = GUICtrlRead($GUI_Panel_Label_PosBX)
	$PosBY = GUICtrlRead($GUI_Panel_Label_PosBY)
EndFunc

Func SaveSettings()
	IniWrite($SaveData, "Filter", "iThresholdAlpha", $Threshold_Alpha)
	IniWrite($SaveData, "Filter", "iThresholdColor", $Threshold_Color)
	IniWrite($SaveData, "Filter", "iFilterW", $Filter_W)
	IniWrite($SaveData, "Filter", "iFilterH", $Filter_H)
	IniWrite($SaveData, "Draw", "iSleepIndex", $SleepIndex)
	IniWrite($SaveData, "Draw", "iSleepInterval", $SleepInterval)
	IniWrite($SaveData, "Draw", "sPathString", $PathString)
	IniWrite($SaveData, "Draw", "bMixPath", BoolToString($MixPath))
	IniWrite($SaveData, "Draw", "bRotatePath", BoolToString($RotatePath))
	IniWrite($SaveData, "Draw", "iRadius", $Radius)
	IniWrite($SaveData, "HotKeys", "sGetMouse", $HotKey_GetMouse)
	IniWrite($SaveData, "HotKeys", "sStartDraw", $HotKey_StartDraw)
	IniWrite($SaveData, "HotKeys", "sShutdown", $HotKey_Shutdown)
	IniWrite($SaveData, "MousePos", "bUseMousePos", BoolToString($UseMousePos))
	IniWrite($SaveData, "MousePos", "iPosAX", $PosAX)
	IniWrite($SaveData, "MousePos", "iPosAY", $PosAY)
	IniWrite($SaveData, "MousePos", "iPosBX", $PosBX)
	IniWrite($SaveData, "MousePos", "iPosBY", $PosBY)
	SaveListData($GUI_Panel_List)
EndFunc

Func RegisterShutdownWithRestart()
	$Restart = True
	HotKeySet($HotKey_Shutdown, "ProcessShutdown")
	TrayTip("Image Draw Info", "Press " & $HotKey_Shutdown & " key to stop.", 5)
EndFunc

Func UnregisterShutdownWithRestart()
	$Restart = False
	HotKeySet($HotKey_Shutdown)
EndFunc

Func ProcessShutdown()
	_GDIPlus_Shutdown()
	MouseUp("primary")
	UpdateSettings()
	SaveSettings()
	If ($Restart) Then
		AddLog($LogLabel_Restart)
		FileClose($LogHandle)
		Run(@ScriptFullPath)
	Else
		AddLog($LogLabel_Exit)
		FileClose($LogHandle)
	EndIf
	Exit
EndFunc
; Script functions / End

; Filename functions / Start
Func FileName($FilePath)
	Return StringRegExpReplace($FilePath, "^.*\\", "")
EndFunc

Func FilePathView($FilePath)
	Return $ViewFolder & "\" & FileName($FilePath) & "_view.bmp"
EndFunc

Func FilePathFilter($FilePath)
	Return $FilterFolder & "\" & FileName($FilePath) & "_filter.bmp"
EndFunc
; Filename functions / End

; Image list functions / Start
Func AddImageToList($FilePath, ByRef $List)
	If (_GUICtrlListBox_FindString($List, $FilePath) > -1) Then
		Return
	EndIf
	_GUICtrlListBox_BeginUpdate($List)
	$Index = _GUICtrlListBox_AddString($List, $FilePath)
	_GUICtrlListBox_EndUpdate($List)
	_GUICtrlListBox_SetCurSel($List, $Index)
	SaveListData($List)
EndFunc

Func MoveCurrentImage(ByRef $List, $Mode)
	$Count = _GUICtrlListBox_GetCount($GUI_Panel_List)
	If ($Count = 0) Then
		Return
	EndIf
	$CompareValue = 0
	$ShiftValue = 0
	Switch $Mode
		Case "Up"
			$CompareValue = 0
			$ShiftValue = -1
		Case "Down"
			$CompareValue = $Count - 1
			$ShiftValue = 1
	EndSwitch
	$Index = _GUICtrlListBox_GetCurSel($GUI_Panel_List)
	If ($Index = $CompareValue) Then
		Return
	EndIf
	_GUICtrlListBox_SwapString($GUI_Panel_List, $Index, ($Index + $ShiftValue))
	_GUICtrlListBox_SetCurSel($GUI_Panel_List, ($Index + $ShiftValue))
EndFunc

Func RemoveImageFromList($FilePath, ByRef $List)
	_GUICtrlListBox_BeginUpdate($List)
	$Index = _GUICtrlListBox_FindString($List, $FilePath)
	_GUICtrlListBox_DeleteString($List, $Index)
	_GUICtrlListBox_EndUpdate($List)
	$Count = _GUICtrlListBox_GetCount($List)
	If ($Count > 0) Then
		If ($Index = $Count) Then
			$Index = $Index - 1
		EndIf
		_GUICtrlListBox_SetCurSel($List, $Index)
	EndIf
	SaveListData($List)
EndFunc

Func ClearList(ByRef $List)
	For $Index = 0 To (_GUICtrlListBox_GetCount($List) - 1) Step 1
		_GUICtrlListBox_DeleteString($List, 0)
	Next
	SaveListData($List)
EndFunc

Func SaveListData(ByRef $List)
	$ListHandle = FileOpen($ImageList, 2)
	FileWriteLine($ImageList, _GUICtrlListBox_GetCurSel($List))
	For $Index = 0 To (_GUICtrlListBox_GetCount($List) - 1) Step 1
		$FilePath = _GUICtrlListBox_GetText($List, $Index)
		If ($FilePath <> "") Then
			FileWriteLine($ImageList, $FilePath)
		EndIf
	Next
	FileClose($ListHandle)
EndFunc

Func LoadListData(ByRef $List)
	$ListHandle = FileOpen($ImageList, 0)
	$Index = Number(FileReadLine($ListHandle, 1))
	_GUICtrlListBox_BeginUpdate($List)
	While True
		$FilePath = FileReadLine($ListHandle)
		If ($FilePath = "") Then
			ExitLoop
		EndIf
		_GUICtrlListBox_AddString($List, $FilePath)
	WEnd
	_GUICtrlListBox_EndUpdate($List)
	FileClose($ListHandle)
	_GUICtrlListBox_SetCurSel($List, $Index)
	ProcessPanelList()
EndFunc
; Image list functions / End

; GUI panel position functions / Start
Func GUI_Panel_X($Scale)
	Return Round($GUI_Panel_W * $Scale)
EndFunc

Func GUI_Panel_Y($Scale)
	Return Round($GUI_Panel_H * $Scale)
EndFunc
; GUI panel position functions / End

; GUI list and buttons functions / Start
Func ProcessPanelList()
	$FilePath = GUICtrlRead($GUI_Panel_List)
	DisplayImageInGUI($FilePath)
EndFunc

Func ProcessPanelButtonAdd()
	$FilePath = FileOpenDialog("Open image", @WorkingDir, "Images (*.jpg;*.jpeg;*.png;*.gif;*.bmp)", 1, "", $GUI_Main)
	If ($FilePath = "") Then
		Return
	EndIf
	AddImageToList($FilePath, $GUI_Panel_List)
	DisplayImageInGUI($FilePath)
EndFunc

Func ProcessPanelButtonUp()
	MoveCurrentImage($GUI_Panel_List, "Up")
EndFunc

Func ProcessPanelButtonDown()
	MoveCurrentImage($GUI_Panel_List, "Down")
EndFunc

Func ProcessPanelButtonRemove()
	$FilePath = GUICtrlRead($GUI_Panel_List)
	RemoveImageFromList($FilePath, $GUI_Panel_List)
	$FilePath = GUICtrlRead($GUI_Panel_List)
	DisplayImageInGUI($FilePath)
EndFunc

Func ProcessPanelButtonClear()
	ClearList($GUI_Panel_List)
	DisplayImageInGUI("")
EndFunc

Func ProcessPanelButtonFilter()
	$FilePath = GUICtrlRead($GUI_Panel_List)
	If ($FilePath = "") Then
		Return
	EndIf
	$Image = _GDIPlus_ImageLoadFromFile($FilePath)
	Local $WidthHeight[2] = FitImageIntoArea($Image, $Filter_W, $Filter_H)
	$Image_W = $WidthHeight[0]
	$Image_H = $WidthHeight[1]
	FilterImageToFile($FilePath, $Image_W, $Image_H)
	_GDIPlus_ImageDispose($Image)
EndFunc

Func ProcessPanelButtonView()
	$FilePath = GUICtrlRead($GUI_Panel_List)
	If ($FilePath = "") Then
		Return
	EndIf
	$FilePathFilter = FilePathFilter($FilePath)
	If (Not FileExists($FilePathFilter)) Then
		Return
	EndIf
	ShellExecute($FilePathFilter)
EndFunc

Func ProcessPanelButtonDrawOne()
	$FilePath = GUICtrlRead($GUI_Panel_List)
	If ($FilePath = "") Then
		Return
	EndIf
	$FilePathFilter = FilePathFilter($FilePath)
	If (Not FileExists($FilePathFilter)) Then
		Return
	EndIf
	Local $LTRB[4] = GetReadyForDraw()
	DrawImage($FilePath, $LTRB[0], $LTRB[1], $LTRB[2], $LTRB[3])
	MsgBox(0, "Image Draw Info", "File done. Press draw button to start again.")
EndFunc

Func ProcessPanelButtonDrawAll()
	$Count = _GUICtrlListBox_GetCount($GUI_Panel_List)
	If ($Count = 0) Then
		Return
	EndIf
	Local $LTRB[4] = GetReadyForDraw()
	For $Index = 0 To ($Count - 1) Step 1
		$FilePath = _GUICtrlListBox_GetText($GUI_Panel_List, $Index)
		If ($FilePath = "") Then
			ContinueLoop
		EndIf
		$FilePathFilter = FilePathFilter($FilePath)
		If (Not FileExists($FilePathFilter)) Then
			ContinueLoop
		EndIf
		DrawImage($FilePath, $LTRB[0], $LTRB[1], $LTRB[2], $LTRB[3])
	Next
	MsgBox(0, "Image Draw Info", "All files done. Press draw button to start again.")
EndFunc
; GUI list and buttons functions / End

; Get two mouse positions to define draw area
Func GetReadyForDraw()
	If ($UseMousePos) Then
		Local $Result[4] = [$PosAX, $PosAY, $PosBX, $PosBY]
		Return $Result
	EndIf
	$MouseLeft = 0
	$MouseTop = 0
	$MouseRight = 0
	$MouseBottom = 0
	HotKeySet($HotKey_GetMouse, "UnlockScript")
	While True
		MsgBox(0, "Image Draw Info", "Press " & $HotKey_GetMouse & " key two times to set draw area")
		LockScript()
		$MousePosA = MouseGetPos()
		$MousePosA_X = $MousePosA[0]
		$MousePosA_Y = $MousePosA[1]
		LockScript()
		$MousePosB = MouseGetPos()
		$MousePosB_X = $MousePosB[0]
		$MousePosB_Y = $MousePosB[1]
		$MouseLeft = _Min($MousePosA_X, $MousePosB_X)
		$MouseTop = _Min($MousePosA_Y, $MousePosB_Y)
		$MouseRight = _Max($MousePosA_X, $MousePosB_X)
		$MouseBottom = _Max($MousePosA_Y, $MousePosB_Y)
		$PosAX = $MouseLeft
		$PosAY = $MouseTop
		$PosBX = $MouseRight
		$PosBY = $MouseBottom
		GUICtrlSetData($GUI_Panel_Label_PosAX, $PosAX)
		GUICtrlSetData($GUI_Panel_Label_PosAY, $PosAY)
		GUICtrlSetData($GUI_Panel_Label_PosBX, $PosBX)
		GUICtrlSetData($GUI_Panel_Label_PosBY, $PosBY)
		$Info = "Area left-top is " & $MouseLeft & ";" & $MouseTop & @CRLF
		$Info = $Info & "Area right-bottom is " & $MouseRight & ";" & $MouseBottom & @CRLF
		$Info = $Info & "Area size is " & ($MouseRight - $MouseLeft) & "x" & ($MouseBottom - $MouseTop) & @CRLF
		$Info = $Info & "Register " & $HotKey_StartDraw & " key for drawing?"
		If (MsgBox(4, "Image Draw Info" , $Info) = 6) Then
			ExitLoop
		EndIf
	WEnd
	Local $LTRB[4] = [$MouseLeft, $MouseTop, $MouseRight, $MouseBottom]
	Return $LTRB
EndFunc

; Image and bitmap functions / Start
Func DisplayImageInGUI($FilePath)
	GUICtrlSetData($GUI_Progress, 0)
	GUICtrlDelete($GUI_Image)
	If ($FilePath = "") Then
		Return
	EndIf
	$Image = _GDIPlus_ImageLoadFromFile($FilePath)
	Local $WidthHeight[2] = FitImageIntoArea($Image, $GUI_Image_W, $GUI_Image_H)
	$GUI_Image_W_Fit = $WidthHeight[0]
	$GUI_Image_H_Fit = $WidthHeight[1]
	$GUI_Image_PosX = ($GUI_Full_W - $GUI_Image_W_Fit) / 2
	$GUI_Image_PosY = ($GUI_Full_H + $GUI_Panel_H - $GUI_Progress_H - $GUI_Image_H_Fit) / 2
	$FilePathView = FilePathView($FilePath)
	$CLSID = _GDIPlus_EncodersGetCLSID("BMP")
	_GDIPlus_ImageSaveToFileEx($Image, $FilePathView, $CLSID)
	$GUI_Image = GUICtrlCreatePic($FilePathView, $GUI_Image_PosX, $GUI_Image_PosY, $GUI_Image_W_Fit, $GUI_Image_H_Fit)
	_GDIPlus_ImageDispose($Image)
EndFunc

Func FilterImageToFile($FilePath, $ImageW, $ImageH)
	If ($FilePath = "") Then
		Return
	EndIf
	$BitmapA = _GDIPlus_BitmapCreateFromFile($FilePath)
	$BitmapB = _GDIPlus_ImageResize($BitmapA, $ImageW, $ImageH)
	$BitmapC = ProcessBitmapUsingThreshold($BitmapB)
	$FilePathFilter = FilePathFilter($FilePath)
	_GDIPlus_ImageSaveToFile($BitmapC, $FilePathFilter)
	_GDIPlus_BitmapDispose($BitmapA)
	_GDIPlus_BitmapDispose($BitmapB)
	_GDIPlus_BitmapDispose($BitmapC)
	ShellExecute($FilePathFilter)
EndFunc

Func ProcessBitmapUsingThreshold($Bitmap)
	$ImageW = _GDIPlus_ImageGetWidth($Bitmap)
	$ImageH = _GDIPlus_ImageGetHeight($Bitmap)
	For $X = 0 To $ImageW - 1 Step 1
		For $Y = 0 To $ImageH - 1 Step 1
			$ColorInt = _GDIPlus_BitmapGetPixel($Bitmap, $X, $Y)
			$ColorBin = Binary($ColorInt)
			$Alpha = Int(BinaryMid($ColorBin, 4, 1))
			$Red = Int(BinaryMid($ColorBin, 3, 1))
			$Green = Int(BinaryMid($ColorBin, 2, 1))
			$Blue = Int(BinaryMid($ColorBin, 1, 1))
			$Shade = ($Red + $Green + $Blue) / 3
			If ($Alpha < $Threshold_Alpha) Or ($Shade > $Threshold_Color) Then
				_GDIPlus_BitmapSetPixel($Bitmap, $X, $Y, $White)
			Else
				_GDIPlus_BitmapSetPixel($Bitmap, $X, $Y, $Black)
			EndIf
		Next
		GUICtrlSetData($GUI_Progress, ($X + 1) / $ImageW * 100)
	Next
	Return $Bitmap
EndFunc

Func FitImageIntoArea($Image, $Area_W, $Area_H)
	$Image_W = _GDIPlus_ImageGetWidth($Image)
	$Image_H = _GDIPlus_ImageGetHeight($Image)
	$Image_Scale_X = $Area_W / $Image_W
	$Image_Scale_Y = $Area_H / $Image_H
	$Image_Scale = _Min($Image_Scale_X, $Image_Scale_Y)
	$Image_W_Fit = Round($Image_W * $Image_Scale)
	$Image_H_Fit = Round($Image_H * $Image_Scale)
	Local $WidthHeight[2] = [$Image_W_Fit, $Image_H_Fit]
	Return $WidthHeight
EndFunc
; Image and bitmap functions / End

; Draw functions / Start
Func DrawImage($FilePath, $Left, $Top, $Right, $Bottom)
	HotKeySet($HotKey_StartDraw, "UnlockScript")
	MsgBox(0, "Image Draw Info", "Ready for the next image:" & @CRLF & $FilePath & @CRLF & "Press " & $HotKey_StartDraw & " key to start")
	LockScript()
	HotKeySet($HotKey_StartDraw)
	$FilePathFilter = FilePathFilter($FilePath)
	$BitmapA = _GDIPlus_BitmapCreateFromFile($FilePathFilter)
	Local $WidthHeight[2] = FitImageIntoArea($BitmapA, $Right - $Left, $Bottom - $Top)
	$Bitmap_W = $WidthHeight[0]
	$Bitmap_H = $WidthHeight[1]
	$BitmapB = _GDIPlus_ImageResize($BitmapA, $Bitmap_W, $Bitmap_H)
	TrayTip("Image Draw Info", "Preparing " & ($Bitmap_W * $Bitmap_H) & " pixels...", 60)
	Local $Pixels[$Bitmap_W][$Bitmap_H]
	For $X = 0 To ($Bitmap_W - 1) Step 1
		For $Y = 0 To ($Bitmap_H - 1) Step 1
			If (_GDIPlus_BitmapGetPixel($BitmapB, $X, $Y) = $Black) Then
				$Pixels[$X][$Y] = 1
			Else
				$Pixels[$X][$Y] = 0
			EndIf
		Next
	Next
	TrayTip("", "", 0)
	_GDIPlus_BitmapDispose($BitmapA)
	_GDIPlus_BitmapDispose($BitmapB)
	$StackSize = 1000
	Local $Stack = CreateStack($StackSize)
	For $X = 0 To ($Bitmap_W - 1) Step 1
		For $Y = 0 To ($Bitmap_H - 1) Step 1
			If $Pixels[$X][$Y] = 1 Then
				MouseMove($Left + $X, $Top + $Y, 0)
				MouseDown("primary")
				$SleepCounter = 0
				DrawArea($Pixels, $Stack, $X, $Y, $Left, $Top, $Bitmap_W, $Bitmap_H)
				MouseUp("primary")
			EndIf
		Next
	Next
EndFunc

Func DrawArea(ByRef $Pixels, ByRef $Stack, $X, $Y, $Left, $Top, $Width, $Height)
	$Continue = False
	$PathLength = StringLen($PathString)
	Local $Path[$PathLength] = StringSplit($PathString, "")
	
	While True
		MouseMove($X + $Left, $Y + $Top, 0)
		$Pixels[$X][$Y] = 2
		
		If ($MixPath) Then
			$Path = MixPath($PathString, $PathLength)
		EndIf
		If ($RotatePath) Then
			$Path = RotatePath($Path, $PathLength)
		EndIf
		
		;;;;;;;;;;;;;;;;;;;
		;; +---+---+---+ ;;
		;; | 1 | 2 | 3 | ;;
		;; +---+---+---+ ;;
		;; | 4 |   | 5 | ;;
		;; +---+---+---+ ;;
		;; | 6 | 7 | 8 | ;;
		;; +---+---+---+ ;;
		;;;;;;;;;;;;;;;;;;;
		
		$Continue = False
		For $Index = 0 To ($PathLength - 1) Step 1
			Switch ($Path[$Index])
				Case 1
					If (($X > 0) And ($Y > 0)) Then
						If ($Pixels[$X - 1][$Y - 1] = 1) Then
							Push($Stack, $X, $Y)
							$X -= 1
							$Y -= 1
							$Continue = True
							ExitLoop
						EndIf
					EndIf
				Case 2
					If ($Y > 0) Then
						If ($Pixels[$X][$Y - 1] = 1) Then
							Push($Stack, $X, $Y)
							$Y -= 1
							$Continue = True
							ExitLoop
						EndIf
					EndIf
				Case 3
					If (($X > 0) And ($Y < 0)) Then
						If ($Pixels[$X + 1][$Y - 1] = 1) Then
							Push($Stack, $X, $Y)
							$X += 1
							$Y -= 1
							$Continue = True
							ExitLoop
						EndIf
					EndIf
				Case 4
					If ($X > 0) Then
						If ($Pixels[$X - 1][$Y] = 1) Then
							Push($Stack, $X, $Y)
							$X -= 1
							$Continue = True
							ExitLoop
						EndIf
					EndIf
				Case 5
					If ($X < ($Width - 1)) Then
						If ($Pixels[$X + 1][$Y] = 1) Then
							Push($Stack, $X, $Y)
							$X += 1
							$Continue = True
							ExitLoop
						EndIf
					EndIf
				Case 6
					If (($X < 0) And ($Y > 0)) Then
						If ($Pixels[$X - 1][$Y + 1] = 1) Then
							Push($Stack, $X, $Y)
							$X -= 1
							$Y += 1
							$Continue = True
							ExitLoop
						EndIf
					EndIf
				Case 7
					If ($Y < ($Height - 1)) Then
						If ($Pixels[$X][$Y + 1] = 1) Then
							Push($Stack, $X, $Y)
							$Y += 1
							$Continue = True
							ExitLoop
						EndIf
					EndIf
				Case 8
					If (($X < ($Width - 1)) And ($Y < ($Height - 1))) Then
						If ($Pixels[$X + 1][$Y + 1] = 1) Then
							Push($Stack, $X, $Y)
							$X += 1
							$Y += 1
							$Continue = True
							ExitLoop
						EndIf
					EndIf
			EndSwitch
		Next
		ProcessSleep()
		DestroyPixelsInRadius($Pixels, $X, $Y, $Radius, $Width - 1, $Height - 1)
		If ($Continue) Then ContinueLoop
		If (Not Pop($Stack, $X, $Y)) Then ExitLoop
	WEnd
EndFunc
; Draw functions / End

; Specific functions to process drawing / Start
Func MixPath($String, $Length)
	Local $Path[$Length]
	For $Index = $Length To 1 Step -1
		$Next = StringMid($String, Random (1, $Index, 1), 1)
		$Path[$Index - 1] = Number($Next)
		$String = StringReplace($String, $Next, "")
	Next
	Return $Path
EndFunc

Func RotatePath($Path, $Length)
	Local $NewPath[$Length]
	For $Index = 1 To ($Length - 1) Step 1
		$NewPath[$Index] = $Path[$Index - 1]
	Next
	$NewPath[0] = $Path[$Length - 1]
	Return $NewPath
EndFunc

Func CreateStack($StackSize)
	Local $Stack[$StackSize + 1][2]
	$Stack[0][0] = 0
	$Stack[0][1] = $StackSize
	Return $Stack
EndFunc

Func Push(ByRef $Stack, $X, $Y)
	$Stack[0][0] += 1
	If ($Stack[0][0] > $Stack[0][1]) Then
		$Stack[0][1] += 1000
		ReDim $Stack[$Stack[0][1] + 1][2]
	EndIf
	$Stack[$Stack[0][0]][0] = $X
	$Stack[$Stack[0][0]][1] = $Y
EndFunc

Func Pop(ByRef $Stack, ByRef $X, ByRef $Y)
	If ($Stack[0][0] < 1) Then
		Return False
	EndIf
	$X = $Stack[$Stack[0][0]][0]
	$Y = $Stack[$Stack[0][0]][1]
	$Stack[0][0] -= 1
	Return True
EndFunc

Func DestroyPixelsInRadius(ByRef $Pixels, $X, $Y, $Radius, $XMax, $YMax)
	For $PosX = ($X - $Radius) To ($X + $Radius) Step 1
		If ($PosX < 0) Or ($PosX > $XMax) Then ContinueLoop
		For $PosY = ($Y - $Radius) To ($Y + $Radius) Step 1
			If ($PosY < 0) Or ($PosY > $YMax) Then ContinueLoop
			$Distance = Sqrt(($X - $PosX)^2 + ($Y - $PosY)^2)
			If ($Distance <= $Radius) Then
				$Pixels[$PosX][$PosY] = 0
			EndIf
		Next
	Next
EndFunc
; Specific functions to process drawing / End
